// Gatekeeper Example File
// ComWalk
// 
// This file demonstrates adding two additional passwords, one of which will
// force the server to make room for the client, invalidating the need for
// reserved slots to be used.

require("gatekeeper")

// The cvar that will act as an additional password
local pass_new = CreateConVar("sv_password_new", "", FCVAR_PROTECTED | FCVAR_NOTIFY)

// The cvar that will make room for a client if they attempt to join
local pass_makeroom = CreateConVar("sv_password_makeroom", "", FCVAR_PROTECTED | FCVAR_NOTIFY)

local function PasswordCheck(name, pass, ip)
	print(Format("Connection attempt! User: '%s' Pass: '%s' IP: '%s'", name, pass, ip))
	
	if pass_new:GetString() == pass and pass_new:GetString() != "" then
		// Since the given password matches the 'new' password
		// and the new password has been set, returning true
		// will prevent the normal password check code
		// from being executed.
		
		return true
	elseif pass_makeroom:GetString() == pass and pass_makeroom:GetString() != "" then
		local players = player.GetAll()
		
		// There is no need to drop anybody if the server isn't full
		if #players == MaxPlayers() then
			local dropme = players[math.random(1, #players)]
			
			// Gatekeeper exposes only one new function to Lua, gatekeeper.Drop,
			// which allows for the server to drop a client with a custom reason.
			// Unlike using RunConsoleCommand to kick the unwanted player, this
			// takes place instantly, and because the maxplayers check takes place
			// after the password check, any player that provides this password to
			// the server will successfully join, without any error message.
			
			gatekeeper.Drop(dropme:UserID(), Format("Auto-kicking to make room for '%s'", name))
		end
		
		// Provided password was valid; force success and prevent
		// the default password check from being executed.
		
		return true
	end
	
	// Returning nil will allow the default password check to take place
	// and, in the case of this example, check sv_password as well.
	// In the event of a custom authentication system in which
	// the default check doesn't need to be run at all, simply
	// returning false here will suffice.
	return
end
hook.Add("PlayerPasswordAuth", "test", PasswordCheck)