#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include "detours.h"
#include "sigscan.h"

#include <interface.h>
#include <netadr.h>
#include <iclient.h>

#include "common/GMLuaModule.h"

GMOD_MODULE(Load, Unload)

ILuaInterface* gLua;

class CSomeEngineClass;
class CDetoured
{
public:
	bool newPasswordCheck(netadr_t* netinfo, const char* Password, const char* Username);
	bool origPasswordCheck(netadr_t* netinfo, const char* Password, const char* Username);
};

bool (CDetoured::* pfPassDetour)(netadr_t*, const char*, const char*);
bool (CDetoured::* pfPassTramp)(netadr_t*, const char*, const char*);

CSomeEngineClass*** ppClientList;
int* pClientListSize;

DETOUR_TRAMPOLINE_EMPTY(bool (CDetoured::origPasswordCheck)(netadr_t* netinfo, const char* pass, const char* user));
bool CDetoured::newPasswordCheck(netadr_t* netinfo, const char* Password, const char* Username)
{
	ILuaObject* Hook = gLua->GetGlobal("hook");	
	ILuaObject* hookCall = Hook->GetMember("Call");

	gLua->Push(hookCall);

	hookCall->UnReference();
	Hook->UnReference();

	gLua->Push("PlayerPasswordAuth");
	gLua->PushNil();
	gLua->Push(Username);
	gLua->Push(Password);
	gLua->Push(netinfo->ToString());

	gLua->Call(5, 1);

	ILuaObject* ret = gLua->GetReturn(0);

	if ( ret->GetType() == GLua::TYPE_BOOL )
		return ret->GetBool();

	if ( !ret->isNil() )
		gLua->ErrorNoHalt("PlayerPasswordAuth hook must return a boolean value\n");

	return origPasswordCheck(netinfo, Password, Username);
}

LUA_FUNCTION(DropPlayer)
{
	gLua->CheckType(1, GLua::TYPE_NUMBER);
	gLua->CheckType(2, GLua::TYPE_STRING);

	if ( *ppClientList == NULL )
		gLua->Error("ppClientList is uninitialized");

	int DropID = gLua->GetNumber(1);

	for (int i=0; i < *pClientListSize; i++)
	{
		IClient* client = (IClient *) ((int)(*ppClientList)[i] + 4);
		
		if ( client->GetUserID() == DropID )
		{
			client->Disconnect(gLua->GetString(2));
			return 0;
		}
	}

	gLua->Error("Invalid userid provided");
	return 0;
}

int Load(lua_State* L)
{
	gLua = Lua();

	CSigScan::sigscan_dllfunc = Sys_GetFactory("engine.dll");
	CSigScan::GetDllMemInfo();

	CSigScan sigClientList;
	sigClientList.Init((unsigned char *) "\x7c\x86\xee\x0c\x8b\x04\xba\x85\xc0\x74\x05\x8d\x70\x04\xEB\x02\x33\xf6\x8b\x06\x8B\x50\x78\x8B\xCE",
		"????xxxxxxxxxxxxxxxxxxxxx", 25);

	if ( !sigClientList.is_set )
		gLua->Error("ClientList signature scan failed!");

	ppClientList = *(CSomeEngineClass ****) sigClientList.sig_addr;
	pClientListSize = (int *) ((int) ppClientList + 0xC);

	CSigScan sigPasswordCheck;
	sigPasswordCheck.Init((unsigned char *) "\x8B\x01\x8B\x50\x6C\x53\xFF\xD2\x8B\xD8\x85\xDB\x75\x06\xB0\x01",
		"xxxxxxxxxxxxxxxx", 16);

	if ( !sigPasswordCheck.is_set )
		gLua->Error("PasswordCheck signature scan failed!");

	pfPassDetour = &CDetoured::newPasswordCheck;
	pfPassTramp = &CDetoured::origPasswordCheck;

	DetourFunctionWithEmptyTrampoline(*(PBYTE *) &pfPassTramp,
		(PBYTE) sigPasswordCheck.sig_addr, *(PBYTE *) &pfPassDetour);

	ILuaObject* gatekeeper = gLua->GetNewTable();
		gatekeeper->SetMember("Drop", DropPlayer);
	gLua->SetGlobal("gatekeeper", gatekeeper);

	gatekeeper->UnReference();

	return 0;
}

int Unload(lua_State* L)
{
	DetourRemove(*(PBYTE*) &pfPassTramp, *(PBYTE*) &pfPassDetour);

	return 0;
}